#pragma once

#include "file_handle.hpp"
#include <memory>
#include <vector>
#include <string>
#include <cstdint>
#include <climits>

namespace mgs {
struct FrameFormat {
	FrameFormat() = default;
	FrameFormat(const FrameFormat&) = default;
	FrameFormat(FrameFormat&&) = default;
	FrameFormat (std::uint32_t x, std::uint32_t y, std::uint32_t fmt);
	~FrameFormat() noexcept = default;

	FrameFormat& operator=(const FrameFormat&) = default;
	FrameFormat& operator=(FrameFormat&&) = default;
	bool operator== (const FrameFormat& o) const noexcept;
	bool operator< (const FrameFormat& o) const noexcept;
	bool operator!=(const FrameFormat& o) const noexcept;

	std::uint32_t width{}, height{};
	std::uint32_t format{};
};

constexpr std::uint32_t fourcc_to_int (const char fourcc[5]) noexcept;

class Camera {
public:
	Camera (std::string path, const FrameFormat& frame_size, bool start_now);
	~Camera() noexcept;
	FileHandle file_handle() const;

	void start_streaming();
	std::vector<std::uint8_t>& read_frame (std::vector<std::uint8_t>& out_buf);
	FrameFormat frame_size() const;
	void set_frame_size (const FrameFormat& new_size);
	unsigned int stride() const;
	const std::string& video_path() const;
	std::vector<FrameFormat> supported_frame_sizes() const;

private:
	struct LocalData;

	void start_opening_device();
	void allocate_buffers();
	void activate_streaming (bool activate);

	std::unique_ptr<LocalData> m_local;
};

inline constexpr std::uint32_t fourcc_to_int (const char fourcc[5]) noexcept {
	return fourcc[0] | (fourcc[1] << CHAR_BIT) | (fourcc[2] << 2 * CHAR_BIT) | (fourcc[3] << 3 * CHAR_BIT);
}
} //namespace mgs
