#include "avcodec.hpp"
extern "C" {
#include <libavcodec/avcodec.h>
} //extern C
#include <stdexcept>
#include <string>

namespace mgs::ffmpeg {
namespace {
::AVPixelFormat get_favourite_format_ifp (::AVCodecContext* s, const ::AVPixelFormat* fmt) {
	::AVPixelFormat fav_format;
	std::copy(
		reinterpret_cast<const char*>(&s->opaque),
		reinterpret_cast<const char*>(&s->opaque) + sizeof(::AVPixelFormat),
		reinterpret_cast<char*>(&fav_format)
	);

	const ::AVPixelFormat* curr_fmt = fmt;
	while (*curr_fmt != -1) {
		if (*curr_fmt == fav_format)
			return fav_format;
		++curr_fmt;
	}
	return ::avcodec_default_get_format(s, fmt);
}
} //unnamed namespace

AVCodec::AVCodec (::AVCodecID id) :
	m_avcodec(::avcodec_find_decoder(id))
{
	if (not m_avcodec)
		throw std::runtime_error("Unable to finde avcodec with id " + std::to_string(id));
}

AVCodec::~AVCodec() noexcept = default;

Decoder AVCodec::make_decoder (unsigned int width, unsigned int height, ::AVPixelFormat dst_format) {
	::AVCodec* const codec = m_avcodec.get();
	Decoder::UniqueAVCodecContext context{::avcodec_alloc_context3(codec)};
	if (not context)
		throw std::runtime_error("Could not allocate video codec context");


	if (codec->capabilities & AV_CODEC_CAP_TRUNCATED)
		context->flags |= AV_CODEC_FLAG_TRUNCATED; // we do not send complete frames

	context->width = width;
	context->height = height;
	context->get_format = &get_favourite_format_ifp;

	static_assert(sizeof(void*) >= sizeof(::AVPixelFormat));
	std::copy(
		reinterpret_cast<const char*>(&dst_format),
		reinterpret_cast<const char*>(&dst_format) + sizeof(::AVPixelFormat),
		reinterpret_cast<char*>(&context->opaque)
	);

	if (::avcodec_open2(context.get(), codec, nullptr) < 0)
		throw std::runtime_error("Could not open codec");

	return {std::move(context), dst_format};
}

} //namespace mgs::ffmpeg
