#include "avdeleter.hpp"
extern "C" {
#include <libavcodec/avcodec.h>
#include <libswscale/swscale.h>
} //extern C
#if !defined(NDEBUG)
#	include <iostream>
#endif

namespace mgs::ffmpeg {
namespace {
void delete_avcodec_object (::AVCodec*) noexcept {
	//no close function for this one
#if !defined(NDEBUG)
	std::cout << "AVCodec object freed\n";
#endif
}

void delete_avcodec_object (::AVCodecContext* ptr) noexcept {
	//avcodec_close(ptr);
	//av_free(ptr);
	avcodec_free_context(&ptr);
#if !defined(NDEBUG)
	std::cout << "AVCodecContext object closed and freed\n";
#endif
}

void delete_avcodec_object (::AVFrame* ptr) noexcept {
	av_frame_free(&ptr);
#if !defined(NDEBUG)
	std::cout << "AVFrame object freed\n";
#endif
}

void delete_avcodec_object (::AVPacket* ptr) noexcept {
	av_packet_free(&ptr);
#if !defined(NDEBUG)
	std::cout << "AVPacket object freed\n";
#endif
}

void delete_avcodec_object (::SwsContext* ptr) noexcept {
	sws_freeContext(ptr);
#if !defined(NDEBUG)
	std::cout << "SwsContext object freed\n";
#endif
}

void delete_avcodec_object (std::uint8_t* ptr) noexcept {
	::av_free(ptr);
}
} //unnamed namespace

template <typename T>
void AVDeleter<T>::operator() (T* ptr) const noexcept {
	delete_avcodec_object(ptr);
}

template struct AVDeleter<::AVCodec>;
template struct AVDeleter<::AVCodecContext>;
template struct AVDeleter<::AVPacket>;
template struct AVDeleter<::AVFrame>;
template struct AVDeleter<::SwsContext>;
template struct AVDeleter<std::uint8_t>;

} // namespace mgs::ffmpeg
