#pragma once

#include "avdeleter.hpp"
#include <cstddef>
#include <cstdint>
extern "C" {
#include <libavutil/pixfmt.h>
} //extern C

namespace mgs::ffmpeg {
class Decoder {
public:
	typedef unique_av_ptr<struct ::AVCodecContext> UniqueAVCodecContext;
	typedef unique_av_ptr<struct ::AVFrame> UniqueAVFrame;
	typedef unique_av_ptr<struct ::AVPacket> UniqueAVPacket;
	typedef unique_av_ptr<struct ::SwsContext> UniqueSwsContext;
	typedef unique_av_ptr<std::uint8_t> UniqueAVMem;

	Decoder (UniqueAVCodecContext&& context, ::AVPixelFormat dst_format);
	~Decoder() noexcept;
	std::size_t decode (const std::uint8_t* input, std::size_t input_size, std::uint8_t* out, std::size_t out_size);
	::AVPixelFormat destination_format() const noexcept;

private:
	void init_sws_context();

	UniqueAVMem m_frame2_mem;
	UniqueAVCodecContext m_context;
	UniqueAVPacket m_packet;
	UniqueAVFrame m_frame1;
	UniqueAVFrame m_frame2;
	UniqueSwsContext m_sws_context;
	::AVPixelFormat m_dst_format;
};
} //namespace mgs::ffmpeg
