#pragma once

#include <cstddef>

namespace mgs {
class FileHandle {
public:
	struct Deleter {
		typedef FileHandle pointer;
		void operator() (pointer fd);
	};

	static const int Mode_RDWR, Mode_RDONLY, Mode_WRONLY, Mode_APPEND,
		Mode_CREAT, Mode_DIRECTORY, Mode_DSYNC, Mode_EXCL, Mode_NOCTTY,
		Mode_NOFOLLOW, Mode_NONBLOCK, Mode_RSYNC, Mode_SYNC, Mode_TRUNC;

	FileHandle(std::nullptr_t = nullptr) : m_handle(0) {}
	FileHandle(int v) : m_handle(v) {}
	FileHandle(const FileHandle&) = default;
	FileHandle(FileHandle&&) = default;
	~FileHandle() noexcept = default;

	FileHandle& operator= (const FileHandle&) = default;
	FileHandle& operator= (FileHandle&&) = default;

	operator bool() const noexcept { return 0 != m_handle; }
	operator int() const noexcept { return m_handle; }
	bool operator!() const noexcept { return !m_handle; }

	friend bool operator==(FileHandle a, FileHandle b) noexcept {
		return a.m_handle == b.m_handle;
	}

	friend bool operator!=(FileHandle a, FileHandle b) noexcept {
		return a.m_handle != b.m_handle;
	}

	friend bool operator== (FileHandle fh, std::nullptr_t) noexcept { return !fh; }
	friend bool operator!= (FileHandle fh, std::nullptr_t) noexcept { return !!fh; }

	static FileHandle open (const char* path, int mode);

private:
	int m_handle;
};
} //namespace mgs
